//
//  iPhoneNativeUiInterface.mm
//  scummvm
//
//  Created by Omer Gilad on 9/27/15.
//
//

#include <UIKit/UIKit.h>
#include <Foundation/Foundation.h>
#include <iostream>

#include "iPhoneNativeUiInterface.h"
#import "iphone_main.h"
#import "iphone_video.h"

#import "Categories/NSStringAdditions.h"
#import "Categories/NSArray+CPPConvertionAdditions.h"
#import "Categories/UIStoryboard+scummVMAdditions.h"

#import "MainViewController.h"
#import "ViewControllers/MenuViewController.h"
#import "ViewControllers/ListDialogViewController.h"
#import "ViewControllers/AlertDialogViewController.h"
#import "ViewControllers/InputDialogViewController.h"
#import "ViewControllers/DeveloperLogoViewController.h"
#import "ViewControllers/ProgressDialogViewController.h"
#import "ViewControllers/iPhoneNativeMPViewController.h"

// Just for logs
#include "common/textconsole.h"

#ifdef USE_GAMECENTER
#import "GameCenterManager.h"
#endif

NSString* const kRefreshSettingsNotification = @"kRefreshSettingsNotification";



UIViewController* TopViewController()
{
    iPhoneMain* appDelegate = (iPhoneMain*)[UIApplication sharedApplication].delegate;
    
//    [[GameCenterManager sharedManager] setDelegate:self];
    
    UIViewController* topViewController = appDelegate.getWindow.rootViewController;
    
    
    while (topViewController.presentedViewController != nil && !topViewController.presentedViewController.isBeingDismissed) {
        topViewController = topViewController.presentedViewController;
    }
    
    topViewController.modalPresentationStyle = UIModalPresentationCurrentContext;
    return topViewController;
}


AVAudioPlayer* musicPlayer()
{
    static AVAudioPlayer* musicBGPlayer = nil;
    
    if (musicBGPlayer == nil) {
        NSURL* resource = [[NSBundle mainBundle] URLForResource:@"bgmusic" withExtension:@"mp3"];
        NSError* error = nil;
        musicBGPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:resource error:&error];
        musicBGPlayer.numberOfLoops = -1;
    }
    
    return musicBGPlayer;
}


AVAudioPlayer* buttonClickPlayer()
{
    static AVAudioPlayer* buttonClickPlayer = nil;
    
    if (buttonClickPlayer == nil) {
        NSURL* resource = [[NSBundle mainBundle] URLForResource:@"click" withExtension:@"mp3"];
        NSError* error = nil;
        buttonClickPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:resource error:&error];
    }
    
    return buttonClickPlayer;
}

void iPhoneNativeUiInterface::showMenu(bool showSplash) {
    
    NSLog(@"IOS:: show MENU ");

    dispatch_async(dispatch_get_main_queue(), ^{
        

        if (showSplash) {
            // play background music
            [musicPlayer() play];
        }
        
        UINavigationController* navController = [[UIStoryboard mainStoryboard] instantiateInitialViewController];
        
        navController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
        
        MenuViewController* menuViewController = [navController.viewControllers firstObject];
        
        menuViewController.showSplash = showSplash == true ? YES:NO;
        menuViewController.nativeUIInterface = this;
        
        [TopViewController() presentViewController:navController animated:NO completion:nil];

        
        //
        // MARK: test code - t7g
        //
        if (GameInfo::getGameType() == GAME_TYPE_T7G)
        {
      //      if (isOpenHouseEnabled())
      //          onOpenHousePressed();
        }
    });
}


 void iPhoneNativeUiInterface::hideMenu() {
 
     NSLog(@"IOS:: HIDE MENU");
     
     dispatch_async(dispatch_get_main_queue(), ^{
         [TopViewController() dismissViewControllerAnimated:NO completion:nil];
         
         // stop background music
         [musicPlayer() stop];

     });
 }

 void iPhoneNativeUiInterface::showTutorialVideo()
{
    // pause background music
    [musicPlayer() pause];
    
    dispatch_async(dispatch_get_main_queue(), ^{
        NSURL* url = [[NSBundle mainBundle] URLForResource:@"tutorial_en" withExtension:@".mp4"];
        
        iPhoneNativeMPViewController* moviePlayerVC = [[iPhoneNativeMPViewController alloc] initWithContentURL:url];
        moviePlayerVC.moviePlayer.controlStyle = MPMovieControlStyleNone;
        
        [TopViewController() presentViewController:moviePlayerVC animated:YES completion:^{
            [[moviePlayerVC moviePlayer] play];
        }];
    });
}

 void iPhoneNativeUiInterface::refreshSettingStates()
{
    // refresh the settings screen
    dispatch_async(dispatch_get_main_queue(), ^{
        [[NSNotificationCenter defaultCenter] postNotificationName:kRefreshSettingsNotification object:nil];

    });
}

void iPhoneNativeUiInterface::showAlertDialog(wstring message, int buttonNum, wstring button1, wstring button2, wstring button3, shared_ptr<Enhanced::AlertDialogListener> listener) {
    
    dispatch_async(dispatch_get_main_queue(), ^{
        
        
        AlertDialogViewController* alertViewController =
        [AlertDialogViewController alertDialogViewControllerWithMessage:[NSString stringWithCPPWString:message]
                                                              buttonNum:buttonNum
                                                                button1:[NSString stringWithCPPWString:button1]
                                                                button2:[NSString stringWithCPPWString:button2]
                                                                button3:[NSString stringWithCPPWString:button3]
                                                                listner:listener];
        
        [TopViewController() presentViewController:alertViewController animated:NO completion:nil];

    });
 }

 void iPhoneNativeUiInterface::showInputDialog(wstring message, string initialInput, wstring negativeButton, wstring positiveButton, shared_ptr<Enhanced::InputDialogListener> listener) {
 
     dispatch_async(dispatch_get_main_queue(), ^{
         
         InputDialogViewController* inputViewController =
         [InputDialogViewController inputeDialogViewContollerWithMessage:[NSString stringWithCPPWString:message]
                                                             intialInput:[NSString stringWithCPPString:initialInput]
                                                          negativeButton:[NSString stringWithCPPWString:negativeButton]
                                                          positiveButton:[NSString stringWithCPPWString:positiveButton]
                                                                listener:listener ];
         
         
         [TopViewController() presentViewController:inputViewController animated:NO completion:nil];
     });

//     });
 }


 void iPhoneNativeUiInterface::showListDialog(wstring message, vector<wstring> items, int buttonNum, wstring negativeButton, wstring positiveButton, shared_ptr<Enhanced::ListDialogListener> listener) {
     
     NSArray* itemsArray = [NSArray arrayWithVectorOfCPPWStrings:items];
     
     dispatch_async(dispatch_get_main_queue(), ^{
         ListDialogViewController* listViewController = [ListDialogViewController listDialogViewControllerWithMessage:[NSString stringWithCPPWString:message]
                                                                                                                items:itemsArray
                                                                                                         buttonNumber:buttonNum
                                                                                                       negativeButton:[NSString stringWithCPPWString:negativeButton]
                                                                                                       positiveButton:[NSString stringWithCPPWString:positiveButton]
                                                                                                              listner:listener];
      
         listViewController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
         
         if (CURRENT_GAME_TYPE == GAME_TYPE_FOTAQ) {
             listViewController.modalPresentationStyle = UIModalPresentationOverCurrentContext;
         }
         
         [TopViewController() presentViewController:listViewController animated:NO completion:^{
             
         }]; 

     });
 }

 void iPhoneNativeUiInterface::showProgressDialog(wstring message) {

     NSLog(@"IOS:: SHOW PROGRESS DIALOG");
     
     NSString* title = [NSString stringWithCPPWString:message];
     
     NSLog(@"IOS:: progress : dispatching to main threaed");
     
     dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
         
//     });
//     dispatch_async(dispatch_get_main_queue(), ^{
         NSLog(@"IOS :: progress on main thread - creating");
         ProgressDialogViewController* progressViewController = [ProgressDialogViewController progressDialogWithMessage:title];
         NSLog(@"IOS :: progress on main thread - presenting");

         [TopViewController() presentViewController:progressViewController animated:NO completion:nil];
         
     });
     
 }

 void iPhoneNativeUiInterface::hideCurrentDialog() {

     NSLog(@"IOS:: HIDE PROGRESS DIALOG");
//
     dispatch_async(dispatch_get_main_queue(), ^{
        
         [TopViewController() dismissViewControllerAnimated:NO completion:nil];
     });

 }


 void iPhoneNativeUiInterface::openAppStoreForRating()
{
}

 Enhanced::Locale iPhoneNativeUiInterface::getLocale() {
     
  //   NSArray* prefferedLanguages = [NSLocale preferredLanguages];
     
    
     NSString *prefix = [[NSLocale preferredLanguages] objectAtIndex:0];
     
     // fix for iOS 9
     NSString *mainLanguageId = nil;
     if ([prefix length] >= 2)
         mainLanguageId = [prefix substringToIndex:2];
     else
         mainLanguageId = prefix;
     
     NSLog(@"Language: %@",mainLanguageId);
     
     if ([mainLanguageId isEqualToString:@"en"]) {
         return Enhanced::Locale::ENGLISH;
     }
     else if ([mainLanguageId isEqualToString:@"fr"])
     {
         return Enhanced::Locale::FRENCH;
     }
     else if ([mainLanguageId isEqualToString:@"es"])
     {
         return Enhanced::Locale::SPANISH;
     }
     else if ([mainLanguageId isEqualToString:@"de"])
     {
         return Enhanced::Locale::GERMAN;
     }
     else if ([mainLanguageId isEqualToString:@"it"])
     {
         return Enhanced::Locale::ITALIAN;
     }
     else if ([mainLanguageId isEqualToString:@"ru"])
     {
         return Enhanced::Locale::RUSSIAN;
     }
     else if ([mainLanguageId isEqualToString:@"cs"])
     {
         return Enhanced::Locale::CZECH;
     }
     else if ([mainLanguageId isEqualToString:@"pl"])
     {
         return Enhanced::Locale::POLISH;
     }
     else if ([mainLanguageId isEqualToString:@"sv"])
     {
         return Enhanced::Locale::SWEDISH;
     }
     else if ([mainLanguageId isEqualToString:@"he"])
     {
         return Enhanced::Locale::HEBREW;
     }
     else //default
     {
         return Enhanced::Locale::ENGLISH;
     }
    
 }

 void iPhoneNativeUiInterface::showSubtitles(wstring text) {
 
     NSString* subs = [NSString stringWithCPPWString:text];

     NSLog(@"IOS:: SHOW SUBTITLES NSString: %@", subs);

     dispatch_async(dispatch_get_main_queue(), ^{
         iPhoneView* view = [((iPhoneMain*)[[UIApplication sharedApplication] delegate]) getView];
         
         [view showSubtitles:subs];
         
     });
 }

void iPhoneNativeUiInterface::hideSubtitles() {
    
    dispatch_async(dispatch_get_main_queue(), ^{
        
        iPhoneView* view = [((iPhoneMain*)[[UIApplication sharedApplication] delegate]) getView];
        
        [view hideSubtitles];
        
        NSLog(@"IOS:: HIDE SUBTITLES");
    });

 }

 void iPhoneNativeUiInterface::showDeveloperLogo1() {
     
     NSLog(@"IOS:: SHOW DEVELOPER LOGO");
     
     // MARK: tony: disable logo
     if (GameInfo::getGameType() == GAME_TYPE_TONYT)
         return;
     
     //workaround the bug
     dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
//     dispatch_async(dispatch_get_main_queue(), ^{
         
         DeveloperLogoViewController* developerLogoViewController = [DeveloperLogoViewController developerLogoWithImage:[UIImage imageNamed:@"developer_logo_1"]];
         
//         UINavigationController* navController = [[UINavigationController alloc] initWithRootViewController:developerLogoViewController];
//         [navController setNavigationBarHidden:YES];
         
         [TopViewController() presentViewController:developerLogoViewController animated:NO completion:nil];
     });

     
 }
 void iPhoneNativeUiInterface::showDeveloperLogo2() {
     
     NSLog(@"IOS:: SHOW DEVELOPER LOGO 2");

     UIImage* logo2 = [UIImage imageNamed:@"developer_logo_2"];
     
     dispatch_async(dispatch_get_main_queue(), ^{
         
         UIViewController* viewController = TopViewController();
         
         if ([viewController  isKindOfClass:[DeveloperLogoViewController class]]) {
            ((DeveloperLogoViewController*)viewController).logoImage = logo2;
         }
         else
         {
             DeveloperLogoViewController* developerLogoViewController = [DeveloperLogoViewController developerLogoWithImage:[UIImage imageNamed:@"developer_logo_2"]];
             
             [TopViewController() presentViewController:developerLogoViewController animated:NO completion:nil];

         }
    });
     
 }

 void iPhoneNativeUiInterface::hideDeveloperLogo() {
     
     NSLog(@"IOS:: HIDE DEVELOPER LOGO");

     dispatch_async(dispatch_get_main_queue(), ^{
         [TopViewController() dismissViewControllerAnimated:NO completion:nil];

     });
 }

void iPhoneNativeUiInterface::sendGameStartAnalytics(SettingInfo& settingInfo)
{
    NSLog(@"IOS:: SEND GAME START STATISTICS");
    
}

void iPhoneNativeUiInterface::onAchievementReached(int achievementId)
{
    NSLog(@"IOS:: ACHIEVEMENT REACHED %d", achievementId);
    
#ifdef USE_GAMECENTER
    
    [[GameCenterManager sharedManager] saveAndReportAchievement:@"achievementId" percentComplete:100 shouldDisplayNotification:true];
    
#endif
    
}

